/*=====================================================================*/
/*                                                                     */
/*       FILE NAME:        fileout.c                                   */
/*                                                                     */
/*       PROGRAM NAME:     fileout  (Prog. 2.6)                        */
/*                                                                     */
/*                                                                     */
/*   Version 1.00                                                      */
/*                                                                     */
/*   (c) Satoru Ozawa and Dieter W. Heermann                           */
/*   Hitachi, Heidelberg July 1994                                     */
/*                                                                     */
/*=====================================================================*/

/***********************************************************************/
/* one-dimensional oscillator, n-particles                             */
/* general interaction                                                 */
/* Runge-Kutta method                                                  */
/* periodic boundary condition                                         */
/* random mass defects                                                 */
/* initial condition: sine function                                    */
/* data output to file                                                 */
/***********************************************************************/

#include <stdio.h>
#include <math.h>
#include "sgl.h"                /* include files for graphics          */

#define  pi       M_PI          /* MP_I is equal to 3.141592.....      */
#define  dt       0.05	        /* time interval of an unit MD step    */	
#define  maxmds   10000         /* maximum number of MD steps          */ 
#define  r        0.3           /* radius of disc                      */
#define  n        64            /* number of particles                 */
#define  fact1   -1.            /* parameter of anharmonic interaction */
#define  fact2    1.           
#define  mbody    1.            /* mass of body particles              */
#define  mdefect  3.            /* mass of defect particles            */
#define  pdefect  0.2           /* defect ratio                        */
#define  kbody    5.            /* force constant                      */
#define  file_name_1   "xv1.data"  /* name of ouput file               */
#define  msamp    20            /* output interval in MD steps         */

void  main(int argc, char *argv[])
{
  float   x[n+2], xnew[n+1];                /* positions of particles  */
  float   v[n+1], vnew[n+1];                /* velocities of particles */
  float   s[n+2], ss[n+2], sss[n+2], ssss[n+1];  /* arrays for Runge-  */
  float   a[n+1], aa[n+1], aaa[n+1], aaaa[n+1];  /* Kutta method       */
  float   m[n+1], k[n+1];                   /* mass and force constant */
  float   e;                                /* total energy            */
  int     mds, i, j, label[n+1];
  double  sin(double rad), cos(double rad), drand48();
  float   A(float xi, float xim1, float xip1,      /* acceralation     */ 
            float mi, float ki, float kim1);
  float   EP(float xi, float xip1, float ki);      /* potential energy */
  FILE    *fh_out;                          /* pointer to output file  */

  if ((fh_out = fopen(file_name_1, "w")) == NULL){  /* open output file */
    fprintf(stderr, "error opening file: %s \n", file_name_1);
    fclose(fh_out);
    return;
  }

 
  sgl_winsize(800,800);        /* display-size in pixels */
  sgl_init();                  /* initialize graphics    */

  for(i=0; i<=n; i++){         /* assignment of defects and masses     */
    if(drand48()<pdefect){     /* drand48(): a random number generator */
      label[i] = 1;
      m[i] = mdefect;
    }
    else{
      label[i] = 0;
      m[i] = mbody;
    }
  }

  for(i=0; i<=n; i++){          /* assignment of force constants */
    k[i] = kbody;
  }

  for(i=1; i<=n; i++){          /* output to file     */
     fprintf(fh_out,"%f, %f, ", m[i], k[i]);
  }

  for(i=1; i<=n; i++){          /* initial conditions */ 
    x[i] =(float)sin(2.*2.*pi*i/n);
    v[i] = 0.;
  }
  x[0] = x[n];
  x[n+1] = x[1];

/*  x[n/2] = 0.1;  */                    /* for chaos analysis !!!  */
 
  for (mds = 1; mds <= maxmds ; mds++){  /* integration of equation */
                                         /* of motion               */
    for(i=1; i<=n; i++){
      a[i] = A(x[i],x[i-1],x[i+1], m[i], k[i], k[i-1]);
      s[i] = v[i];
    }
    s[0] = s[n];               /* periodic boundary conditions */
    s[n+1] = s[1];
    
    for(i=1; i<=n; i++){
      aa[i] = A(x[i]+s[i]*dt/2., x[i-1]+s[i-1]*dt/2.,x[i+1]+s[i+1]*dt/2.,
              m[i], k[i], k[i-1]);
      ss[i] = v[i] + a[i]*dt/2.;
    }
    ss[0] = ss[n];             /* periodic boundary conditions */
    ss[n+1] = ss[1];

    for(i=1; i<=n; i++){
      aaa[i] = A(x[i]+ss[i]*dt/2., x[i-1]+ss[i-1]*dt/2., 
                 x[i+1]+ss[i+1]*dt/2., m[i], k[i], k[i-1]);
      sss[i] = v[i] + aa[i]*dt/2.;
    }
    sss[0] = sss[n];           /* periodic boundary conditions */
    sss[n+1] = sss[1];

    for(i=1; i<=n; i++){
      aaaa[i] = A(x[i]+sss[i]*dt, x[i-1]+sss[i-1]*dt, 
                  x[i+1]+sss[i+1]*dt, m[i], k[i], k[i-1]);
      ssss[i] = v[i] + aaa[i]*dt;
    }

    for(i=1; i<=n; i++){
      xnew[i] = x[i] + (s[i]+2.*ss[i]+2.*sss[i]+ssss[i])*dt/6.;
      vnew[i] = v[i] + (a[i]+2.*aa[i]+2.*aaa[i]+aaaa[i])*dt/6.;
    }
    
    for(i=1; i<=n; i++){
      x[i] = xnew[i];
      v[i] = vnew[i];
    }
    x[0] = x[n];               /* periodic boundary conditions */
    x[n+1] = x[1];

    for(i=1; i<=n; i++){                     /* display motion */
      if (label[i]==0){
        sgl_disc(0.5+(0.2+x[i]*0.1)*(float)sin(2.*pi*i/n),
                 0.4+(0.2+x[i]*0.1)*(float)cos(2.*pi*i/n), r/n);
        sgl_disc(1./n*i-0.5/n, x[i]*0.1+0.8, r/n);
      }
      else{
        sgl_disc(0.5+(0.2+x[i]*0.1)*(float)sin(2.*pi*i/n),
                 0.4+(0.2+x[i]*0.1)*(float)cos(2.*pi*i/n), r/n);
        sgl_circle(0.5+(0.2+x[i]*0.1)*(float)sin(2.*pi*i/n),
                 0.4+(0.2+x[i]*0.1)*(float)cos(2.*pi*i/n), r*1.5/n);
        sgl_disc(1./n*i-0.5/n, x[i]*0.1+0.8, r/n);
        sgl_circle(1./n*i-0.5/n, x[i]*0.1+0.8, r*1.5/n);
      }
    }
    sgl_circle(0.5, 0.4, 0.2);
    sgl_line(0., 0.8, 1., 0.8);
    sgl_display();
    
    printf("x[0]=%f, x[1]=%f, x[2]=%f,..x[n]=%f, x[n+1]=%f\n",
           x[0],x[1],x[2],x[n],x[n+1]);

    e = 0.;                               /* total energy */
    for(i=1; i<=n; i++){ 
      e +=  m[i]/2.*v[i]*v[i] + EP(x[i], x[i+1], k[i]);
    }
 
    printf("mds=%d, e=%f \n", mds, e);

    if(mds%msamp == 0){                   /* output to file */
      for(i=1; i<=n; i++){ 
        fprintf(fh_out,"%f, %f, ", x[i], v[i]);
      }
    }

  }
  fclose(fh_out);
  j = getchar();
}



/***********************************************************************/
/* anharmonic interaction                                              */
/***********************************************************************/

float   A(float xi, float xim1, float xip1,
          float mi, float ki, float kim1)
{
  float a;

  a = -kim1/mi*(xi-xim1) - fact1*kim1/mi*(xi-xim1)*(xi-xim1)
      -fact2*kim1/mi*(xi-xim1)*(xi-xim1)*(xi-xim1)
      +ki/mi*(xip1-xi) + fact1*ki/mi*(xip1-xi)*(xip1-xi)
      +fact2*ki/mi*(xip1-xi)*(xip1-xi)*(xip1-xi);
  return a;
}



/***********************************************************************/
/* potential function                                                  */
/***********************************************************************/

float   EP(float xi, float xip1, float ki)
{
  float ep;
    
  ep = ki/2.*(xip1-xi)*(xip1-xi)
      +fact1*ki/3.*(xip1-xi)*(xip1-xi)*(xip1-xi)
      +fact2*ki/4.*(xip1-xi)*(xip1-xi)*(xip1-xi)*(xip1-xi);
  return ep;
}

