/*==========================================================================*/
/*                                                                          */
/*       Polymer grafted to a surface                                       */
/*                                                                          */
/*       PROGRAM NAME:      polymer                                         */
/*                                                                          */
/*       CALLING SEQUENCE:                                                  */
/*                                                                          */
/*         Main program without command line arguments.                     */
/*                                                                          */
/*       RETURN:                                                            */
/*                                                                          */
/*         Type int       - Retrun code                                     */
/*                                                                          */
/*   Version 1.00                                                           */
/*                                                                          */
/*   Remarks:                                                               */
/*     -  All coordinates are scaled by the excluded volume sigma           */
/*     -  Polymer is contraint to the positive half space                   */
/*     -  First polymer is grafted at (0,0,0)                               */
/*                                                                          */
/*   Try upon first using the program these parameters                      */
/*    N = 10;                                                               */
/*    l0 = 0.4;                                                             */
/*    SpringBond = 100;                                                     */
/*    SpringAngle = 10;                                                     */
/*    Theta = 3.1415 / 4.0;                                                 */
/*    kTemperature = 1.0;                                                   */
/*    max_sweeps = 10000;                                                   */
/*    seed = 4711;                                                          */
/*                                                                          */
/*   (c) Satoru Ozawa and Dieter W. Heermann                                */
/*   Hitachi, Heidelberg July 1994                                          */
/*                                                                          */
/*==========================================================================*/

/* ==== Constants ==== */

# define eps         0.01       /* displacement for the monomers      */
# define WindowSize  500        /* Size in pixle of the graphics      */

/* ==== Include files for this program ==== */

# include <math.h>
# include <stdio.h>

# include "sgl.h"               /* include for the graphics library */
# include "r250.c"              /* random number generator          */


/*--------------------------------------------------------------------------*/
/*                         Main program                                     */
/*--------------------------------------------------------------------------*/
#ifdef  __STDC__
void    main(int argc, char *argv[])
#else
void    main(argc,argv)
        int     argc;
        char    *argv[];
#endif
 {
    /*---------------------------------------------------------- */
    /*                     Declarations                          */
    /*---------------------------------------------------------- */

    /* ====  Area Declarations ==== */

    int     N;                        /* Length of the chain                */
    int     seed, i, sweep, max_sweeps;
    int     index, MonomerIndex;
    int     sample;
    int     ReturnCode;

    float   SpringBond, SpringAngle, Theta, l0, CosTheta0, kTemperature;
    float   DisplacementX, DisplacementY;

    float   xim, yim;
    float   xip, yip;
    float   xi, yi;
    float   skp_ab, abs_a, abs_b, abs_ab;
    float   const_1;
    float   h1,h2;
    float   rd;
    float   NewPotential, OldPotential;
    float   DeltaEnergy;
    float   w;
    float   scale;

    float   *random;
    float   *x, *y;

    int     mf[251];

    /*---------------------------------------------------------- */
    /*                End of declares                            */
    /*---------------------------------------------------------- */
 
    /*---------------------------------------------------------- */
    /*      Start with setting up procedures                     */
    /*---------------------------------------------------------- */

    /* ====  Print the header for this test  ==== */

    printf("===============================================\n");
    printf("======>  Polymer Grafted to a Surface  <=======\n");
    printf("===============================================\n");
    printf("\n\n");
    printf("(c) Satoru Ozawa and Dieter W. Heermann \n");
    printf("Version 1.00 (1994) \n");
    printf("\n\n");

    printf("Length of the chain                = ");
    scanf("%d",&N);
    printf("Equilibrium length of a bond       = ");
    scanf("%f",&l0);
    printf("Coupling strength for a bond       = ");
    scanf("%f",&SpringBond);
    printf("Equilibrium bond angle (rad)       = ");
    scanf("%f",&Theta);
    printf("Coupling strength for a bond angle = ");
    scanf("%f",&SpringAngle);
    printf("Temperature                        = ");
    scanf("%f",&kTemperature);
    printf("Number of Monte Carlo steps        = ");
    scanf("%d",&max_sweeps);
    printf("Seed for Random Number Generator   = ");
    scanf("%d",&seed);
 
    /* ==== Grep the storage for the random numbers.        ==== */

    random = (float *) calloc(3*N, sizeof(float));
    if ( random == NULL ) {
        printf("Not enough memory");
        exit(-1);
    }
    x = (float *) calloc(N, sizeof(float));
    if ( x == NULL ) {
        printf("Not enough memory");
        exit(-1);
    }
    y = (float *) calloc(N, sizeof(float));
    if ( y == NULL ) {
        printf("Not enough memory");
        exit(-1);
    }
 
    /* ==== set up the random number generator              ==== */

    seed        = (seed << 2) + 1;
    ReturnCode = init_r250(seed,mf);

    /* ==== starting configuration ==== */

    CosTheta0     = cos( Theta );
    DisplacementY = l0 * sin( Theta );
    x[0] = 0;
    y[0] = 0;
    for( i=1; i < N; i++ ) {
        /* x[i] =  (i%2 == 0) ? 0, l0 * CosTheta0; */
        if ( i % 2 == 0 ) x[i] = 0;
        else x[i] = l0 * CosTheta0;
        y[i] =  i * DisplacementY;
    }

    /* ==== initialize the graphics ==== */

    sgl_winsize(WindowSize,WindowSize); 
    sgl_init(); 
    sgl_color_background(SGL_BLACK); 
    scale = N * l0;
    for(i=0; i < N; i++) {
        if ( i > 0 ) {
            sgl_line((float) (l0 - x[i-1])*l0, (float) y[i-1]/scale,
                     (float) (l0 - x[i])*l0,   (float) y[i]/scale); 
            }
    }
    sgl_display(); 
    i = getchar();

    /*---------------------------------------------------------- */
    /*                M o n t e  C a r l o                       */
    /*---------------------------------------------------------- */

    for(sweep=0; sweep < max_sweeps; sweep++) {

        if ( sweep % 100 == 0 ) printf(" sweep = %d\n",sweep);
        /* select at random */
        sample       = N;
        index        = 0;
        ReturnCode  = r250(5*N,random,mf);
        while ( sample ) {
            sample --;

            /* choose monomer and displace */
            MonomerIndex  = (int) (random[index++] * N);
            DisplacementX = eps * ( 2.0 * random[index++] - 1.0 );
            DisplacementY = eps * ( 2.0 * random[index++] - 1.0 );

            /* check whether we still satify the constraint */
            if ( y[MonomerIndex] + DisplacementY > 0.0 ) {

                /* compute the changes in energy: */
                /* First the old energy           */

                OldPotential = 0.0;
                if ( (MonomerIndex > 0) && (MonomerIndex < N-1) ) {
                    /* ----  inner monomer   ----*/

                    /* bond length potential */
                    xi = x[MonomerIndex] - x[MonomerIndex+1];
                    yi = y[MonomerIndex] - y[MonomerIndex+1];
                    rd         = sqrt(xi*xi + yi*yi);
                    OldPotential += (rd - l0) * (rd - l0);

                    xi = x[MonomerIndex-1] - x[MonomerIndex];
                    yi = y[MonomerIndex-1] - y[MonomerIndex];
                    rd         = sqrt(xi*xi + yi*yi);
                    OldPotential += (rd - l0) * (rd - l0);
                }
                else {
                    /* ---- end monomer ---- */

                    /* bond length potential */
                    if ( MonomerIndex == 0 ) {
                        xi = x[MonomerIndex] - x[MonomerIndex+1];
                        yi = y[MonomerIndex] - y[MonomerIndex+1];
                        rd         = sqrt(xi*xi + yi*yi);
                    }
                    else {
                        xi = x[MonomerIndex-1] - x[MonomerIndex];
                        yi = y[MonomerIndex-1] - y[MonomerIndex];
                        rd         = sqrt(xi*xi + yi*yi);
                    }
                    OldPotential += (rd - l0) * (rd - l0);
                }

                OldPotential *= SpringBond;

                if ( MonomerIndex > 1 ) {
                    /* ---- angle between (i-2,i) ---- */

                    xim = x[MonomerIndex-2] - x[MonomerIndex-1];
                    yim = y[MonomerIndex-2] - y[MonomerIndex-1];

                    xip = x[MonomerIndex] - x[MonomerIndex-1];
                    yip = y[MonomerIndex] - y[MonomerIndex-1];

                    skp_ab = xim * xip  +  yim * yip;
                    abs_a  = sqrt(xim * xim  +  yim * yim);
                    abs_b  = sqrt(xip * xip  +  yip * yip);
                    abs_ab = abs_a * abs_b;

                    const_1 = (skp_ab / abs_ab) - CosTheta0;
                    OldPotential += 0.5 * SpringAngle * const_1 * const_1;

                } /* end (i-2,i) */
 
                if ( MonomerIndex < (N-2) ) {
                    /* ---- angle between (i,i+2) ---- */

                    xim = x[MonomerIndex] - x[MonomerIndex+1];
                    yim = y[MonomerIndex] - y[MonomerIndex+1];

                    xip = x[MonomerIndex+2] - x[MonomerIndex+1];
                    yip = y[MonomerIndex+2] - y[MonomerIndex+1];

                    skp_ab = xim * xip  +  yim * yip;
                    abs_a  = sqrt(xim * xim  +  yim * yim);
                    abs_b  = sqrt(xip * xip  +  yip * yip);
                    abs_ab = abs_a * abs_b;

                    const_1 = (skp_ab / abs_ab) - CosTheta0;
                    OldPotential += 0.5 * SpringAngle * const_1 * const_1;

                } /* end (i,i+2) */

                /* ---- Now the calculation between (i-1,i+1) ---- */

                if ( (MonomerIndex > 0) && ( MonomerIndex < N-1) ) {
                     xim = x[MonomerIndex-1] - x[MonomerIndex];
                     yim = y[MonomerIndex-1] - y[MonomerIndex];

                     xip = x[MonomerIndex+1] - x[MonomerIndex];
                     yip = y[MonomerIndex+1] - y[MonomerIndex];

                     skp_ab = xim * xip  +  yim * yip;
                     abs_a  = sqrt(xim * xim  +  yim * yim);
                     abs_b  = sqrt(xip * xip  +  yip * yip);
                     abs_ab = abs_a * abs_b;

                     const_1 = (skp_ab / abs_ab) - CosTheta0;
                     OldPotential += 0.5 * SpringAngle * const_1 * const_1;
                }

                /* compute the changes in energy: */
                /* Second the new energy          */

                NewPotential = 0.0;
                if ( (MonomerIndex > 0) && (MonomerIndex < N-1) ) {
                    /* ----  inner monomer   ----*/

                    /* bond length potential */
                    xi = (x[MonomerIndex] + DisplacementX) - x[MonomerIndex+1];
                    yi = (y[MonomerIndex] + DisplacementY) - y[MonomerIndex+1];
                    rd         = sqrt(xi*xi + yi*yi);
                    NewPotential += (rd - l0) * (rd - l0);

                    xi = x[MonomerIndex-1] - (x[MonomerIndex] + DisplacementX);
                    yi = y[MonomerIndex-1] - (y[MonomerIndex] + DisplacementY);
                    rd         = sqrt(xi*xi + yi*yi);
                    NewPotential += (rd - l0) * (rd - l0);
                }
                else {
                    /* ---- end monomer ---- */

                    /* bond length potential */
                    if ( MonomerIndex == 0 ) {
                        xi = (x[MonomerIndex] + DisplacementX) - x[MonomerIndex+1];
                        yi = (y[MonomerIndex] + DisplacementY) - y[MonomerIndex+1];
                        rd         = sqrt(xi*xi + yi*yi);
                    }
                    else {
                        xi = x[MonomerIndex-1] - (x[MonomerIndex] + DisplacementX);
                        yi = y[MonomerIndex-1] - (y[MonomerIndex] + DisplacementY);
                        rd         = sqrt(xi*xi + yi*yi);
                    }
                    NewPotential += (rd - l0) * (rd - l0);
                }

                NewPotential *= SpringBond;

                if ( MonomerIndex > 1 ) {
                    /* ---- angle between (i-2,i) ---- */

                    xim = x[MonomerIndex-2] - x[MonomerIndex-1];
                    yim = y[MonomerIndex-2] - y[MonomerIndex-1];

                    xip = (x[MonomerIndex] + DisplacementX) - x[MonomerIndex-1];
                    yip = (y[MonomerIndex] + DisplacementY) - y[MonomerIndex-1];

                   skp_ab = xim * xip  +  yim * yip;
                   abs_a  = sqrt(xim * xim  +  yim * yim);
                   abs_b  = sqrt(xip * xip  +  yip * yip);
                   abs_ab = abs_a * abs_b;

                   const_1 = (skp_ab / abs_ab) - CosTheta0;
                   NewPotential += 0.5 * SpringAngle * const_1 * const_1;

                } /* end (i-2,i) */
 
                if ( MonomerIndex < (N-2) ) {
                    /* ---- angle between (i,i+2) ---- */

                    xim = (x[MonomerIndex] + DisplacementX) - x[MonomerIndex+1];
                    yim = (y[MonomerIndex] + DisplacementY) - y[MonomerIndex+1];

                    xip = x[MonomerIndex+2] - x[MonomerIndex+1];
                    yip = y[MonomerIndex+2] - y[MonomerIndex+1];

                    skp_ab = xim * xip  +  yim * yip;
                    abs_a  = sqrt(xim * xim  +  yim * yim);
                    abs_b  = sqrt(xip * xip  +  yip * yip);
                    abs_ab = abs_a * abs_b;

                    const_1 = (skp_ab / abs_ab) - CosTheta0;
                    NewPotential += 0.5 * SpringAngle * const_1 * const_1;

                } /* end (i,i+2) */

                /* ---- Now the calculation between (i-1,i+1) ---- */

                if ( (MonomerIndex > 0) && ( MonomerIndex < (N-1)) ) {
                    xim = x[MonomerIndex-1] - (x[MonomerIndex] + DisplacementX);
                    yim = y[MonomerIndex-1] - (y[MonomerIndex] + DisplacementY);

                    xip = x[MonomerIndex+1] - (x[MonomerIndex] + DisplacementX);
                    yip = y[MonomerIndex+1] - (y[MonomerIndex] + DisplacementY);

                    skp_ab = xim * xip  +  yim * yip;
                    abs_a  = sqrt(xim * xim  +  yim * yim);
                    abs_b  = sqrt(xip * xip  +  yip * yip);
                    abs_ab = abs_a * abs_b;

                    const_1 = (skp_ab / abs_ab) - CosTheta0;
                    NewPotential += 0.5 * SpringAngle * const_1 * const_1;
                }

                DeltaEnergy = NewPotential - OldPotential;
                w           = exp( - DeltaEnergy / kTemperature );
                if ( random[index++] < w ) {
                    /* accept the new monomer placement */
                    x[MonomerIndex] += DisplacementX;
                    y[MonomerIndex] += DisplacementY;
                }
            } /* contraint */
        } /* sample monomer loop */

    scale = N * l0;

    sgl_color_background(SGL_BLACK); 
    for(i=0; i < N; i++) {
        if ( i > 0 ) {
            sgl_line((float) (5*l0 + x[i-1])/scale, (float) y[i-1]/scale,
                     (float) (5*l0 + x[i])/scale,   (float) y[i]/scale); 
            }
    }
    sgl_display(); 
    if ( sweep % 100 == 0 ) {
        printf(" hold: press return");
        getchar();
    }

    } /* Monte Carlo loop */
} /* end of program */

