/*=====================================================================*/
/*                                                                     */
/*       FILE NAME:        rungen.c                                    */
/*                                                                     */
/*       PROGRAM NAME:     rungen   (prog. 2.4)                        */
/*                                                                     */
/*                                                                     */
/*   Version 1.00                                                      */
/*                                                                     */
/*   (c) Satoru Ozawa and Dieter W. Heermann                           */
/*   Hitachi, Heidelberg July 1994                                     */
/*                                                                     */
/*=====================================================================*/

/***********************************************************************/
/* one-dimensional oscillator, n-particles                             */
/* general interaction                                                 */
/* Runge-Kutta method                                                  */
/* periodic boundary condition                                         */
/* initial condition: pulse                                            */
/***********************************************************************/

#include <stdio.h>
#include <math.h>
#include "sgl.h"                /* include files for graphics          */

#define  pi      M_PI           /* MP_I is equal to 3.141592.....      */
#define  dt      0.05           /* time interval of an unit MD step    */
#define  maxmds  1000           /* maximum number of MD steps          */
#define  r       0.3            /* radius of disc                      */
#define  n       64             /* number of particles                 */
#define  fact1  -0.5            /* parameter of anharmonic interaction */
#define  fact2   0.25
#define  mbody   1.             /* mass of body particles              */
#define  kbody   1.             /* force constant                      */

void  main(int argc, char *argv[])
{
  float   x[n+2], xnew[n+1];                /* positions of particles  */
  float   v[n+1], vnew[n+1];                /* velocities of particles */
  float   s[n+2], ss[n+2], sss[n+2], ssss[n+1];  /* arrays for Runge-  */
  float   a[n+1], aa[n+1], aaa[n+1], aaaa[n+1];  /* Kutta method       */
  float   m[n+1], k[n+1];                   /* mass and force constant */
  float   e;                                /* total energy            */
  int     mds, i;
  double  sin(double rad), cos(double rad);
  float   A(float xi, float xim1, float xip1,      /* acceralation     */
            float mi, float ki, float kim1); 
  float   EP(float xi, float xip1, float ki);      /* potential energy */
 
  sgl_winsize(800,800);   /* display size in pixels */
  sgl_init();             /* initialize graphics    */

  for(i=0; i<=n; i++){     /* assignment of masses */
                           /* and force constants  */
    m[i] = mbody;
    k[i] = kbody;
  }

  for(i=1; i<=n; i++){     /* initial conditions   */ 
    x[i] = 0.;
    v[i] = 0.;
  }
  x[n/2] = +1.;
  x[n/2+1] = +1.;
  x[0] = x[n];
  x[n+1] = x[1];
 
  for (mds = 1; mds <= maxmds ; mds++){   /* integration of equation */
                                          /* of motion               */
    for(i=1; i<=n; i++){
      a[i] = A(x[i],x[i-1],x[i+1], m[i], k[i], k[i-1]);
      s[i] = v[i];
    }
    s[0] = s[n];               /* periodic boundary conditions */
    s[n+1] = s[1];
    
    for(i=1; i<=n; i++){
      aa[i] = A(x[i]+s[i]*dt/2., x[i-1]+s[i-1]*dt/2.,x[i+1]+s[i+1]*dt/2.,
              m[i], k[i], k[i-1]);
      ss[i] = v[i] + a[i]*dt/2.;
    }
    ss[0] = ss[n];             /* periodic boundary conditions */
    ss[n+1] = ss[1];

    for(i=1; i<=n; i++){
      aaa[i] = A(x[i]+ss[i]*dt/2., x[i-1]+ss[i-1]*dt/2., 
                 x[i+1]+ss[i+1]*dt/2., m[i], k[i], k[i-1]);
      sss[i] = v[i] + aa[i]*dt/2.;
    }
    sss[0] = sss[n];           /* periodic boundary conditions */
    sss[n+1] = sss[1];

    for(i=1; i<=n; i++){
      aaaa[i] = A(x[i]+sss[i]*dt, x[i-1]+sss[i-1]*dt, 
                  x[i+1]+sss[i+1]*dt, m[i], k[i], k[i-1]);
      ssss[i] = v[i] + aaa[i]*dt;
    }

    for(i=1; i<=n; i++){
      xnew[i] = x[i] + (s[i]+2.*ss[i]+2.*sss[i]+ssss[i])*dt/6.;
      vnew[i] = v[i] + (a[i]+2.*aa[i]+2.*aaa[i]+aaaa[i])*dt/6.;
    }
    
    for(i=1; i<=n; i++){
      x[i] = xnew[i];
      v[i] = vnew[i];
    }
    x[0] = x[n];               /* periodic boundary conditions */
    x[n+1] = x[1];

    for(i=1; i<=n; i++){                      /* display motion */
      sgl_disc(0.5+(0.2+x[i]*0.1)*(float)sin(2.*pi*i/n),
               0.4+(0.2+x[i]*0.1)*(float)cos(2.*pi*i/n), r/n);
      sgl_disc(1./n*i-0.5/n, x[i]*0.1+0.8, r/n);
    }
    sgl_circle(0.5, 0.4, 0.2);
    sgl_line(0., 0.8, 1., 0.8);
    sgl_display();
 
    printf("x[0]=%f, x[1]=%f, x[2]=%f,..x[n]=%f, x[n+1]=%f\n", 
           x[0],x[1],x[2],x[n],x[n+1]);
    
    e = 0.;                                  /* total energy */
    for(i=1; i<=n; i++){                
      e = e + m[i]/2.*v[i]*v[i]
            + EP(x[i], x[i+1], k[i]);
    }
    printf("mds=%d, e=%f \n", mds, e);
  }
  i = getchar();         /* hold final picture until hit of any key */
}



/***********************************************************************/
/* anharmonic interaction                                              */
/***********************************************************************/

float   A(float xi, float xim1, float xip1,
          float mi, float ki, float kim1)
{
  float a;

  a = -kim1/mi*(xi-xim1) - fact1*kim1/mi*(xi-xim1)*(xi-xim1)
      -fact2*kim1/mi*(xi-xim1)*(xi-xim1)*(xi-xim1)
      +ki/mi*(xip1-xi) + fact1*ki/mi*(xip1-xi)*(xip1-xi)
      +fact2*ki/mi*(xip1-xi)*(xip1-xi)*(xip1-xi);
  return a;
}



/***********************************************************************/
/* potential function                                                  */
/***********************************************************************/

float   EP(float xi, float xip1, float ki)
{
  float ep;
    
  ep = ki/2.*(xip1-xi)*(xip1-xi)
      +fact1*ki/3.*(xip1-xi)*(xip1-xi)*(xip1-xi)
      +fact2*ki/4.*(xip1-xi)*(xip1-xi)*(xip1-xi)*(xip1-xi);
  return ep;
}
